import aiohttp
from lcwc import Client
from lcwc.agencies.agencyresolver import AgencyResolver
from lcwc.web.incident import WebIncident as Incident
from lcwc.web.parser import WebParser


class WebClient(Client):
    """Client for scraping the live incident page"""

    URL = "https://www.lcwc911.us/live-incident-list"
    """ The URL of the live incident page """

    def __init__(self, agency_resolver: AgencyResolver = AgencyResolver()) -> None:
        self.agency_resolver = agency_resolver
        self.parser = WebParser()

    @property
    def name(self) -> str:
        """Returns the name of the client"""
        return "WebClient"

    async def get_incidents(
        self, session: aiohttp.ClientSession, timeout: int = 10
    ) -> list[Incident]:
        """Fetches the live incident page and returns a list of incidents

        :param session: The aiohttp session to use
        :param timeout: The timeout in seconds
        :return: A list of incidents
        :rtype: list[Incident]
        """
        html = None
        async with session.get(self.URL, timeout=timeout) as resp:
            if resp.status == 200:
                html = await resp.read()
            else:
                raise Exception(f"Unable to fetch live incident page: {resp.status}")

        return self.parser.parse(html, self.agency_resolver)



from dataclasses import dataclass, field
import datetime

from lcwc.category import IncidentCategory
from lcwc.unit import Unit


@dataclass
class Incident:
    """Represents an incident"""

    """ The category of the incident """
    category: IncidentCategory

    """ The date and time of the incident """
    date: datetime

    """ The description of the incident """
    description: str

    """ The location of the incident location """
    municipality: str

    """ The intersection of the incident location """
    intersection: str

    """ A list of units responding to the incident """
    units: list[Unit]



__title__ = 'lcwc'
__author__ = 'Nate Shoffner'
__license__ = 'MIT'
__copyright__ = 'Copyright 2023-present Nate Shoffner'
__version__ = '0.11'

from .agencies.agency import Agency
from .agencies.agencyclient import AgencyClient
from .category import IncidentCategory
from .client import Client
from .incident import Incident
from .unit import Unit


from dataclasses import dataclass
from lcwc.incident import Incident


@dataclass
class FeedIncident(Incident):
    """Represents an incident from the live incident feed"""

    """Returns the guid of the incident"""
    guid: str



import re
from lcwc.agencies.agencyresolver import AgencyResolver
from lcwc.category import IncidentCategory
from lcwc.unit import Unit


class UnitParserException(Exception):
    pass


class UnitParser:
    @staticmethod
    def parse_unit(
        unit_str: str,
        category: IncidentCategory,
        agency_resolver: AgencyResolver = AgencyResolver(),
    ) -> Unit:
        """Parses the given unit string and returns a Unit object

        :param unit_str: The unit string to parse
        :param category: The category for for the unit
        :param agency_resolver: The agency resolver to use for agency lookups (required for shorthand unit names)
        :return: A Unit object
        :rtype: Unit
        """

        is_shorthand = " " not in unit_str
        if is_shorthand:
            return UnitParser.__parse_short_name(unit_str, category, agency_resolver)
        else:
            return UnitParser.__parse_name(unit_str, category, agency_resolver)

    @staticmethod
    def __parse_name(
        unit_str: str, category: IncidentCategory, agency_resolver: AgencyResolver
    ) -> Unit:
        u = Unit(full_name=unit_str)

        if unit_str == "PENDING":
            u.pending = True
            return u

        # Ex: QRS 10
        # Ex: MEDIC 56-4
        # Ex: AMB 89-1 CHESTER
        match = re.match(
            r"^([a-zA-Z ]+) ([0-9]+)(?:-([0-9]+))?(?: ([a-zA-Z ]+))?", unit_str
        )
        if match is None:
            raise UnitParserException(f"Unable to parse unit name: {unit_str}")

        name, station_id, identifier, county_name = match.groups()

        # TODO sometimes the station id is not present in the unit name (ex: "RESCUE 905")
        # this might just be a misunderstanding in how these units are named
        # for now, if the identifier is not present, then the station id is the identifier
        if identifier is None:
            identifier = station_id
            station_id = None

        u.name = name
        u.station_id = station_id
        u.id = int(identifier)

        if county_name:
            u.county_name = county_name
            u.out_of_county = True

        agency = agency_resolver.get_agency(station_id, category)
        if agency:
            u.agency = agency

        return u

    @staticmethod
    def __parse_short_name(
        unit_str: str, category: IncidentCategory, agency_resolver: AgencyResolver
    ):
        """Identifies the agency associated with the given unit short name"""

        """
        # non-regex alternative for short name parsing
        unit_groups = ["".join(x) for _, x in itertools.groupby(unit.name, key=str.isdigit)]
        abbr = unit_groups[0]
        identifer = unit_groups[1]
        county_abbr = unit_groups[2] if len(unit_groups) > 2 else None
        """

        u = Unit(full_name=unit_str, is_shorthand=True)

        if unit_str == "PENDING":
            u.pending = True
            return u

        # Ex: "ENG531" -> "ENG", "531", None
        # Ex: "AMB891CHE" -> "AMB", "891", "CHE"
        match = re.match(r"([a-zA-Z]+)([0-9]+)([a-zA-Z]*)", unit_str)
        if match is None:
            raise UnitParserException(f"Unable to parse unit name: {unit_str}")

        abbr, identifer, county_abbr = match.groups()

        u.name = abbr
        u.id = identifer

        # identifer is a collapsed string of both the station id + unit id
        # remember to work with id as a string since leading zeros are important

        # iterate through all the characters in the id and build the agency id and suffix
        for i in range(len(identifer)):
            agency_id_builder = identifer[: i + 1]
            agency_suffix = identifer[i + 1 :]
            # agency id is padded with zeros to 2 digits (ex: "07")
            padded_id = str(agency_id_builder.zfill(2))
            a = agency_resolver.get_agency(padded_id, category)
            if a is not None:
                u.station_id = padded_id
                u.agency = a
            if agency_suffix:
                u.id = int(agency_suffix)

        if county_abbr:
            u.county_name = county_abbr
            u.out_of_county = True

        return u



from pydantic import BaseModel
from lcwc.category import IncidentCategory


class Agency(BaseModel):
    """Represents a stationed agency"""

    """ The category of the agency """
    category: IncidentCategory

    """ The station number of the agency """
    station_number: str

    """ The name of the agency """
    name: str

    """ The URL of the agency """
    url: str

    """ The address of the agency """
    address: str

    """ The city of the agency """
    city: str

    """ The state of the agency """
    state: str

    """ The zip code of the agency """
    zip_code: int

    """ The phone number of the agency """
    phone: str

    def __eq__(self, other):
        if not isinstance(other, Agency):
            return False
        return (
            self.category == other.category
            and self.station_number == other.station_number
        )



from dataclasses import dataclass
from typing import Optional

from lcwc.agencies.agency import Agency


@dataclass(eq=False)
class Unit:
    """Represents a unit responding to an incident"""

    """ The fully-qualified name of the unit """
    full_name: str = None

    """ The fully-qualified name is using shorthand notation """
    is_shorthand: Optional[bool] = False

    """ The name of the unit """
    name: Optional[str] = None

    """ The agency associated with the unit """
    agency: Optional[Agency] = None

    """ The ID of the agency associated with the unit """
    station_id: Optional[str] = None

    """ The unit ID """
    id: Optional[int] = None

    """ The unit originates from outside of the county """
    out_of_county: Optional[bool] = False

    """ The name of the county the unit originates from """
    county_name: Optional[str] = None

    """ The unit is pending assignment """
    pending: Optional[bool] = False

    def __eq__(self, other):
        if not isinstance(other, Unit):
            return NotImplemented
        return self.full_name == other.full_name

    def __str__(self) -> str:
        if self.full_name:
            return self.full_name
        return self.__repr__()



from enum import Enum


class IncidentCategory(str, Enum):
    """Represents the category of an incident"""

    FIRE = "Fire"
    """ Fire incident """
    MEDICAL = "Medical"
    """ Medical incident """
    TRAFFIC = "Traffic"
    """ Traffic incident """
    UNKNOWN = "Unknown"
    """ Unknown incident """



import re
from lcwc.agencies import ALL_KNOWN_AGENCIES
from lcwc.agencies.agency import Agency
from lcwc.agencies.exceptions import OutOfCountyException, PendingUnitException
from lcwc.category import IncidentCategory
from lcwc.incident import Incident
from lcwc.unit import Unit


class AgencyResolver:
    """Collection of dispatch and various lookup methods"""

    def __init__(self, load_known: bool = True):
        self.agencies = ALL_KNOWN_AGENCIES if load_known else []

    def add_agency(self, agency: Agency):
        self.agencies.append(agency)

    def remove_agency(self, agency: Agency):
        self.agencies.remove(agency)

    def get_agency(self, station_id: str, category: IncidentCategory) -> Agency:
        """Attempts to find the agency associated with the given station id and category within the list of agencies provided"""
        for agency in self.agencies:
            if agency.station_number == station_id and agency.category == category:
                return agency

    def get_agencies(self, category: IncidentCategory) -> list[Agency]:
        return [agency for agency in self.agencies if agency.category == category]

    def get_all_agencies(self) -> list[Agency]:
        return self.agencies

    def get_unit_agency(self, unit: Unit, category: IncidentCategory) -> Agency:
        """Attempts to find the agency associated with the given unit and category within the list of agencies provided"""

        if unit.is_shorthand():
            return self.__unit_short_name_to_agency(unit, category)
        else:
            return self.__unit_name_to_agency(unit, category)
